local TOCNAME,BOM=...
local L = setmetatable({}, {__index = function (t, k)  
	if BOM.L and BOM.L[k] then 
		return BOM.L[k]
	else
		return "["..k.."]"
	end	
end})
BUFFOMAT_ADDON=BOM

--[[
0.50
- Repair hasResurrection detection - should not resurrect somebody which has already resurrected
- Option "Only buff self"
- Option "Ignore in raid group x"

0.40
- Chinese translation by wellcat
- add "Find Treasure" tracking spell
- fallback resurrection, try to resurrect a "ghost"-player
- Use now Rank and better spell-detection


0.30 
- resurrection optimizations
- buff handling optimizations
- fix bug with mages
- bug fixes


0.20
- repair distance-check (list) / Sort order List
- Wait for SPELLS_CHANGED Event before scanning for spells
- party update only on special events
- Option "Don't use group buff"
- minimap-button indicates now missing buffs
- better combat - detection / indicates
- disable button while global cool-down
- disable button while casting
- disable button while combat
- many small changes and bug fixes

0.10
- first release

-- TODO
- Gruppen ignorieren
- about
-- only selfbuff

]]--

--BOM.IsDebug=true

BOM.Version=GetAddOnMetadata(TOCNAME, "Version") 
BOM.Title=GetAddOnMetadata(TOCNAME, "Title") 

BOM.Icon="Ability_Druid_ChallangingRoar"
BOM.IconOff="Ability_Druid_DemoralizingRoar"
BOM.FullIcon="Interface\\ICONS\\Ability_Druid_ChallangingRoar"
BOM.TxtEscapeIcon="|T%s:0:0:0:0:64:64:4:60:4:60|t"

BOM.MSGPREFIX="BOM:"
BOM.MACRONAME="Buff'o'mat"

local updateOn={"UNIT_AURA", "PLAYER_REGEN_ENABLED", "PLAYER_REGEN_DISABLED", "PLAYER_ENTER_COMBAT", "PLAYER_LEAVE_COMBAT", "PLAYER_ENTERING_WORLD", "READY_CHECK", "LOADING_SCREEN_DISABLED",  "PLAYER_ALIVE", "PLAYER_UNGHOST", "INCOMING_RESURRECT_CHANGED"}
local PartyChangeEvent={ "GROUP_JOINED", "GROUP_ROSTER_UPDATE", "RAID_ROSTER_UPDATE","GROUP_LEFT"}
local SpellChangedEvent={"SPELLS_CHANGED","LEARNED_SPELL_IN_TAB"}

local function PopupDB(db,var)
	return L["Cbox"..var],false,db,var
end

function BOM.Popup(minimap)
	BOM.PopupDynamic:Wipe()
	if minimap then
		BOM.PopupDynamic:AddItem(L.BtnOpen,nil,false,BOM.ShowWindow)
		BOM.PopupDynamic:AddItem("",nil,true)
	end
	
	for i,spell in ipairs(BOM.Spells) do 
		BOM.PopupDynamic:AddItem(spell.singleLink or spell.single,"keep",BOM.DBChar.Spells,spell.ConfigID)
	end
	

	BOM.PopupDynamic:AddItem("",true)
	BOM.PopupDynamic:AddItem(L["Cboxshowminimapbutton"],false,BOM.DB.Minimap,"visible")
	BOM.PopupDynamic:AddItem(L["CboxLockMinimapButton"],false,BOM.DB.Minimap,"lock")
	BOM.PopupDynamic:AddItem(L["CboxLockMinimapButtonDistance"],false,BOM.DB.Minimap,"lockDistance")
	BOM.PopupDynamic:AddItem("",true)
	BOM.PopupDynamic:AddItem(PopupDB(BOM.DB,"AutoOpen"))
	BOM.PopupDynamic:AddItem(PopupDB(BOM.DB,"DeathBlock"))
	BOM.PopupDynamic:AddItem(PopupDB(BOM.DB,"IgnoreClass"))
	BOM.PopupDynamic:AddItem(PopupDB(BOM.DB,"NoGroupBuff"))
	BOM.PopupDynamic:AddItem(PopupDB(BOM.DB,"OnlySelf"))
	BOM.PopupDynamic:SubMenu(L["HeaderIgnoreGroup"],"subGroup")
	for i=1,8 do
		BOM.PopupDynamic:AddItem(i,"keep",BOM.IgnoreGroup,i)
	end
	BOM.PopupDynamic:SubMenu()
	
	BOM.PopupDynamic:AddItem("",true)
	BOM.PopupDynamic:AddItem(L.BtnSettings,false,BOM.Options.Open,BOM.Options)
	BOM.PopupDynamic:AddItem("",true)
	BOM.PopupDynamic:AddItem(L["BtnCancel"],false)
	
	
	BOM.PopupDynamic:Show()
end

function BOM.BtnClose()
	BOM.HideWindow()
end

function BOM.BtnSettings()
	BOM.Popup()
end

function BOM.BtnMacro()
	PickupMacro(BOM.MACRONAME)
end

function BOM.ScrollMessage(self,delta)
	self:SetScrollOffset(self:GetScrollOffset() + delta*5);
	self:ResetAllFadeTimes()
end

function BOM.OptionsUpdate()
	BOM.UpdateScan()
	BOM.MinimapButton:UpdatePosition()
	BOM.Options:DoCancel() 
end

local function BOM_CheckBox(Var,Init)
	BOM.Options:AddCheckBox(BOM.DB,Var,Init,L["Cbox"..Var])
end

function BOM.OptionsInit()
	BOM.Options.Init(
		function()-- doOk
			BOM.Options:DoOk() 
			BOM.OptionsUpdate()
		end,
		function()-- doCancel
			BOM.Options:DoCancel() 
			BOM.OptionsUpdate()
		end,
		function()--doDefault
			BOM.Options:DoDefault() 
			BOM.DB.Minimap.position=50
			BOM.ResetWindow()			
			BOM.OptionsUpdate()
		end
		)
	BOM.Options:AddPanel(BOM.Title)
	BOM.Options:AddVersion('|cff00c0ff' .. BOM.Version .. "|r")
	
	BOM.Options:AddCheckBox(BOM.DB.Minimap,"visible",true,L["Cboxshowminimapbutton"])
	BOM.Options:AddCheckBox(BOM.DB.Minimap,"lock",false,L["CboxLockMinimapButton"])
	BOM.Options:AddCheckBox(BOM.DB.Minimap,"lockDistance",false,L["CboxLockMinimapButtonDistance"])
	BOM.Options:AddSpace()
	BOM_CheckBox("AutoOpen",true)
	BOM_CheckBox("DeathBlock",true)
	BOM_CheckBox("IgnoreClass",false)
	BOM_CheckBox("NoGroupBuff",false)
	BOM_CheckBox("OnlySelf",false)
	
	
	BOM.Options:AddSpace()
end

local OptionsInsertSpells_once
function BOM.OptionsInsertSpells()
	if OptionsInsertSpells_once then return end
	OptionsInsertSpells_once=true

	for i,spell in ipairs(BOM.Spells) do 
		BOM.Options:AddCheckBox(BOM.DBChar, spell.ConfigID, spell.default, spell.singleLink or spell.single)
	end
	
	BOM.Options:AddPanel(L.PanelSupportedSpells,false,true)
	BOM.Options:Indent(10)
	--local txt=""
	
	for i,spell in ipairs(BOM.SpellList) do
		if type(spell)=="table" then
			if spell.groupId then
				BOM.Options:AddText((spell.singleLink or spell.single).." / "..(spell.groupLink or spell.group))
				--txt=txt..(spell.singleLink or spell.single).." / "..(spell.groupLink or spell.group).."\n"
			else
				BOM.Options:AddText((spell.singleLink or spell.single or spell.ConfigID))
				--txt=txt..(spell.singleLink or spell.single).."\n"
			end
		else
			BOM.Options:Indent(-10)
			if LOCALIZED_CLASS_NAMES_MALE [spell] then
				BOM.Options:AddCategory(LOCALIZED_CLASS_NAMES_MALE[spell])
				--txt=txt..LOCALIZED_CLASS_NAMES_MALE[spell].."\n"
			else
				BOM.Options:AddCategory(L["Header_"..spell])
				--txt=txt..L["Header_"..spell].."\n"
			end
			
			BOM.Options:Indent(10)
		end
	end
	--BOM.DB.DEBUGTXT=txt
	
end


function BOM.Init()
	function SetDefault(db,var,init)
		if db[var]==nil then db[var]=init end
	end

	if not buffOmatDB then buffOmatDB = {} end
	if not buffOmatDB.Minimap then buffOmatDB.Minimap = {} end
	if not buffOmatDBChar then buffOmatDBChar = {} end
	if not buffOmatDBChar.Spells then buffOmatDBChar.Spells = {} end
	
	
	BOM.DB=buffOmatDB
	BOM.DBChar=buffOmatDBChar
	
	local x, y, w, h = BOM.DB.X, BOM.DB.Y, BOM.DB.Width, BOM.DB.Height	
	if not x or not y or not w or not h then
		BOM.SaveWindowPosition()
	else
		BuffOmat_MainWindow:ClearAllPoints()
		BuffOmat_MainWindow:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
		BuffOmat_MainWindow:SetWidth(w)
		BuffOmat_MainWindow:SetHeight(h)
	end
	
	BOM.Tool.EnableSize(BuffOmat_MainWindow,8,nil,function() BOM.SaveWindowPosition()	end)
	
	-- slash command
	local function doDBSet(DB,var,value)
		if value==nil then
			DB[var]= not DB[var]
		elseif tContains({"true","1","enable"},value) then
			DB[var]=true
		elseif tContains({"false","0","disable"},value) then
			DB[var]=false
		end
		DEFAULT_CHAT_FRAME:AddMessage(BOM.MSGPREFIX.."Set "..var.." to "..tostring(DB[var]))
		BOM.OptionsUpdate()
	end
	BOM.Tool.SlashCommand({"/bom","/buffomat"},{
		{"debug","",{
				{"buff","",BOM.DebugBuffs},
			},
		},
		{"spellbook",L["SlashSpellBook"],BOM.GetSpells},
		{"update",L["SlashUpdate"],BOM.UpdateScan},
		{"close",L["SlashClose"],BOM.HideWindow},
		{"reset",L["SlashReset"],BOM.ResetWindow},
		{"",L["SlashOpen"],BOM.ShowWindow},
	})
	
	BuffOmat_ListTab_MessageFrame:SetFading(false);
	--if RollTrackerClassicDB.LootTracker.SmallFont then
		BuffOmat_ListTab_MessageFrame:SetFontObject(GameFontNormalSmall);
	--else
	--	BuffOmat_ListTab_MessageFrame:SetFontObject(GameFontNormal);
	--end
	BuffOmat_ListTab_MessageFrame:SetJustifyH("LEFT");
	BuffOmat_ListTab_MessageFrame:SetHyperlinksEnabled(true);
	BuffOmat_ListTab_MessageFrame:SetTextCopyable(true);
	BuffOmat_ListTab_MessageFrame:Clear()
	BuffOmat_ListTab_MessageFrame:SetMaxLines(100)
	
	
	BuffOmat_ListTab_Button:SetAttribute("type", "macro")
	BuffOmat_ListTab_Button:SetAttribute("macro", BOM.MACRONAME)	

	BOM.Tool.OnUpdate(BOM.UpdateTimer)
	
	BOM.PopupDynamic=BOM.Tool.CreatePopup(BOM.OptionsUpdate)
		
	BOM.MinimapButton.Init(BOM.DB.Minimap,BOM.FullIcon,
		function(self,button)
			if button=="LeftButton" then 
				BOM.ToggleWindow()
			else
				BOM.Popup(true)
			end
		end
		,BOM.Title)	
	
	BuffOmat_MainWindow_Title:SetText(string.format(BOM.TxtEscapeIcon,BOM.FullIcon)..BOM.Title)
	--BuffOmat_ListTab_Button:SetText(L["BtnGetMacro"])
	
	BOM.OptionsInit()
	BOM.PartyUpdateNeeded=true
	BOM.RepeatUpdate=false
	
	BOM.IgnoreGroup={}
end

local LastUpdateTimer
function BOM.UpdateTimer()

	if BOM.CheckCoolDown then
		local cdtest=GetSpellCooldown(BOM.CheckCoolDown)
		if cdtest==0 then
			BOM.CheckCoolDown=nil
			BOM.UpdateScan()
		end
	end

	if (BOM.ForceUpdate or BOM.RepeatUpdate) and  LastUpdateTimer~=time() then
		BOM.UpdateScan()
		
		LastUpdateTimer=time()
		--print("forced update")
	end
end


local function Event_ADDON_LOADED(arg1)
	if arg1 == TOCNAME then
		BOM.Init()
	end
	BOM.Tool.AddDataBrocker(BOM.FullIcon, 
		function(self,button)
			if button=="LeftButton" then 
				BOM.ToggleWindow()
			else
				BOM.Popup(true)
			end
		end)
end

local function Event_UNIT_POWER_UPDATE(arg1,arg2)
	--UNIT_POWER_UPDATE: "unitTarget", "powerType"
	if arg2=="MANA" and UnitIsUnit(arg1, "player") then
		if (BOM.ManaLimit or 0)<=(UnitPower("player",0) or 0) then
			BOM.ForceUpdate=true
		end
	end
end

local function Event_Generic_Update()
	BOM.ForceUpdate=true
end

local function Event_SPELLS_CHANGED()
	BOM.GetSpells()
	BOM.ForceUpdate=true
	BOM.OptionsInsertSpells()
end

local function Event_PartyChanged()
	BOM.PartyUpdateNeeded=true
	BOM.ForceUpdate=true
end

local function Event_UNIT_SPELLCAST_START(uid)
	if UnitIsPlayer(uid) then
		--print("casting")
		BOM.PlayerCasting=true
		BOM.ForceUpdate=true
	end
end

local function Event_UNIT_SPELLCAST_STOP(uid)
	if UnitIsPlayer(uid) then
		--print("endcasting")
		BOM.PlayerCasting=false
		BOM.ForceUpdate=true
	end
end

function BOM.OnLoad()
	--print("internal load")
	BuffOmat_MainWindow:SetMinResize(150,75)

	
	BOM.Tool.EnableMoving(BuffOmat_MainWindow,BOM.SaveWindowPosition)
	
	BOM.Tool.RegisterEvent("ADDON_LOADED",Event_ADDON_LOADED)
	BOM.Tool.RegisterEvent("UNIT_POWER_UPDATE",Event_UNIT_POWER_UPDATE)
	BOM.Tool.RegisterEvent("UNIT_SPELLCAST_START",Event_UNIT_SPELLCAST_START)
	BOM.Tool.RegisterEvent("UNIT_SPELLCAST_STOP",Event_UNIT_SPELLCAST_STOP)
	for i,event in ipairs(SpellChangedEvent) do
		BOM.Tool.RegisterEvent(event,Event_SPELLS_CHANGED)
	end
	for i,event in ipairs(PartyChangeEvent) do
		BOM.Tool.RegisterEvent(event,Event_PartyChanged)
	end
	for i,event in ipairs(updateOn) do
		BOM.Tool.RegisterEvent(event,Event_Generic_Update)
	end
	
end

function BOM.HideWindow()
	BuffOmat_MainWindow:Hide()
end

function BOM.ShowWindow()
	BuffOmat_MainWindow:Show()
end

function BOM.WindowVisible()
	return BuffOmat_MainWindow:IsVisible()
end

function BOM.ToggleWindow()
	if BuffOmat_MainWindow:IsVisible() then
		BOM.HideWindow()
	else
		BOM.UpdateScan()
		BOM.ShowWindow()
	end
end


function BOM.SaveWindowPosition()
	BOM.DB.X = BuffOmat_MainWindow:GetLeft()
	BOM.DB.Y = BuffOmat_MainWindow:GetTop()
	BOM.DB.Width = BuffOmat_MainWindow:GetWidth()
	BOM.DB.Height = BuffOmat_MainWindow:GetHeight()
end

function BOM.ResetWindow()
	BuffOmat_MainWindow:ClearAllPoints()
	BuffOmat_MainWindow:SetPoint("Center", UIParent, "Center", 0, 0)
	BuffOmat_MainWindow:SetWidth(200)
	BuffOmat_MainWindow:SetHeight(200)
	BOM.SaveWindowPosition()
	BOM.ShowWindow(1)
end



local function WhoRequest(name)
	DEFAULT_CHAT_FRAME.editBox:SetText("/who " .. name) 
	ChatEdit_SendText(DEFAULT_CHAT_FRAME.editBox)
end

local function WhisperRequest(name)
	ChatFrame_OpenChat("/w " .. name .." ")
end


function BOM.EnterHyperlink(link)
	GameTooltip:SetOwner(BuffOmat_MainWindow,"ANCHOR_PRESERVE")
	GameTooltip:ClearLines()
	GameTooltip:SetHyperlink(link)
	GameTooltip:Show()
end
function BOM.LeaveHyperlink()
	GameTooltip:Hide()
end
function BOM.ClickHyperlink(link)
	local part=BOM.Tool.Split(link,":")
	if part[1]=="unit" then
		if IsShiftKeyDown() then
			WhoRequest(part[3])
			--SendWho( req.name )
		else
			WhisperRequest(part[3])
		end
	end
end